@@ -46,6 +46,7 @@ struct ovs_chassis_cfg {
     const char *datapath_type;
     const char *encap_csum;
     const char *cms_options;
+    const char *monitor_all;
     const char *chassis_macs;
 
     /* Set of encap types parsed from the 'ovn-encap-type' external-id. */
@@ -120,6 +121,12 @@ get_cms_options(const struct smap *ext_ids)
     return smap_get_def(ext_ids, "ovn-cms-options", "");
 }
 
+static const char *
+get_monitor_all(const struct smap *ext_ids)
+{
+    return smap_get_def(ext_ids, "ovn-monitor-all", "false");
+}
+
 static const char *
 get_encap_csum(const struct smap *ext_ids)
 {
@@ -237,6 +244,7 @@ chassis_parse_ovs_config(const struct ovsrec_open_vswitch_table *ovs_table,
     ovs_cfg->datapath_type = get_datapath_type(br_int);
     ovs_cfg->encap_csum = get_encap_csum(&cfg->external_ids);
     ovs_cfg->cms_options = get_cms_options(&cfg->external_ids);
+    ovs_cfg->monitor_all = get_monitor_all(&cfg->external_ids);
     ovs_cfg->chassis_macs = get_chassis_mac_mappings(&cfg->external_ids);
 
     if (!chassis_parse_ovs_encap_type(encap_type, &ovs_cfg->encap_type_set)) {
@@ -261,67 +269,76 @@ chassis_parse_ovs_config(const struct ovsrec_open_vswitch_table *ovs_table,
 }
 
 static void
-chassis_build_external_ids(struct smap *ext_ids, const char *bridge_mappings,
+chassis_build_other_config(struct smap *config, const char *bridge_mappings,
                            const char *datapath_type, const char *cms_options,
-                           const char *chassis_macs, const char *iface_types,
-                           bool is_interconn)
+                           const char *monitor_all, const char *chassis_macs,
+                           const char *iface_types, bool is_interconn)
 {
-    smap_replace(ext_ids, "ovn-bridge-mappings", bridge_mappings);
-    smap_replace(ext_ids, "datapath-type", datapath_type);
-    smap_replace(ext_ids, "ovn-cms-options", cms_options);
-    smap_replace(ext_ids, "iface-types", iface_types);
-    smap_replace(ext_ids, "ovn-chassis-mac-mappings", chassis_macs);
-    smap_replace(ext_ids, "is-interconn", is_interconn ? "true" : "false");
+    smap_replace(config, "ovn-bridge-mappings", bridge_mappings);
+    smap_replace(config, "datapath-type", datapath_type);
+    smap_replace(config, "ovn-cms-options", cms_options);
+    smap_replace(config, "ovn-monitor-all", monitor_all);
+    smap_replace(config, "iface-types", iface_types);
+    smap_replace(config, "ovn-chassis-mac-mappings", chassis_macs);
+    smap_replace(config, "is-interconn", is_interconn ? "true" : "false");
 }
 
 /*
  * Returns true if any external-id doesn't match the values in 'chassis-rec'.
  */
 static bool
-chassis_external_ids_changed(const char *bridge_mappings,
+chassis_other_config_changed(const char *bridge_mappings,
                              const char *datapath_type,
                              const char *cms_options,
+                             const char *monitor_all,
                              const char *chassis_macs,
                              const struct ds *iface_types,
                              bool is_interconn,
                              const struct sbrec_chassis *chassis_rec)
 {
     const char *chassis_bridge_mappings =
-        get_bridge_mappings(&chassis_rec->external_ids);
+        get_bridge_mappings(&chassis_rec->other_config);
 
     if (strcmp(bridge_mappings, chassis_bridge_mappings)) {
         return true;
     }
 
     const char *chassis_datapath_type =
-        smap_get_def(&chassis_rec->external_ids, "datapath-type", "");
+        smap_get_def(&chassis_rec->other_config, "datapath-type", "");
 
     if (strcmp(datapath_type, chassis_datapath_type)) {
         return true;
     }
 
     const char *chassis_cms_options =
-        get_cms_options(&chassis_rec->external_ids);
+        get_cms_options(&chassis_rec->other_config);
 
     if (strcmp(cms_options, chassis_cms_options)) {
         return true;
     }
 
+    const char *chassis_monitor_all =
+        get_monitor_all(&chassis_rec->other_config);
+
+    if (strcmp(monitor_all, chassis_monitor_all)) {
+        return true;
+    }
+
     const char *chassis_mac_mappings =
-        get_chassis_mac_mappings(&chassis_rec->external_ids);
+        get_chassis_mac_mappings(&chassis_rec->other_config);
     if (strcmp(chassis_macs, chassis_mac_mappings)) {
         return true;
     }
 
     const char *chassis_iface_types =
-        smap_get_def(&chassis_rec->external_ids, "iface-types", "");
+        smap_get_def(&chassis_rec->other_config, "iface-types", "");
 
     if (strcmp(ds_cstr_ro(iface_types), chassis_iface_types)) {
         return true;
     }
 
     bool chassis_is_interconn =
-        smap_get_bool(&chassis_rec->external_ids, "is-interconn", false);
+        smap_get_bool(&chassis_rec->other_config, "is-interconn", false);
     if (chassis_is_interconn != is_interconn) {
         return true;
     }
@@ -340,37 +357,51 @@ chassis_tunnels_changed(const struct sset *encap_type_set,
                         const char *encap_csum,
                         const struct sbrec_chassis *chassis_rec)
 {
-    size_t encap_type_count = 0;
+    struct sset chassis_rec_encap_type_set =
+        SSET_INITIALIZER(&chassis_rec_encap_type_set);
+    bool  changed = false;
 
-    for (int i = 0; i < chassis_rec->n_encaps; i++) {
-
-        if (!sset_contains(encap_type_set, chassis_rec->encaps[i]->type)) {
+    for (size_t i = 0; i < chassis_rec->n_encaps; i++) {
+        if (strcmp(chassis_rec->name, chassis_rec->encaps[i]->chassis_name)) {
             return true;
         }
-        encap_type_count++;
+
+        if (!sset_contains(encap_type_set, chassis_rec->encaps[i]->type)) {
+            changed = true;
+            break;
+        }
+        sset_add(&chassis_rec_encap_type_set, chassis_rec->encaps[i]->type);
 
         if (!sset_contains(encap_ip_set, chassis_rec->encaps[i]->ip)) {
-            return true;
+            changed = true;
+            break;
         }
 
         if (strcmp(smap_get_def(&chassis_rec->encaps[i]->options, "csum", ""),
                    encap_csum)) {
-            return true;
+            changed = true;
+            break;
         }
     }
 
-    size_t tunnel_count =
-        sset_count(encap_type_set) * sset_count(encap_ip_set);
+    if (!changed) {
+        size_t tunnel_count =
+            sset_count(encap_type_set) * sset_count(encap_ip_set);
 
-    if (tunnel_count != chassis_rec->n_encaps) {
-        return true;
+        if (tunnel_count != chassis_rec->n_encaps) {
+            changed = true;
+        }
     }
 
-    if (sset_count(encap_type_set) != encap_type_count) {
-        return true;
+    if (!changed) {
+        if (sset_count(encap_type_set) !=
+                sset_count(&chassis_rec_encap_type_set)) {
+            changed = true;
+        }
     }
 
-    return false;
+    sset_destroy(&chassis_rec_encap_type_set);
+    return changed;
 }
 
 /*
@@ -419,58 +450,79 @@ chassis_build_encaps(struct ovsdb_idl_txn *ovnsb_idl_txn,
  * chassis_state.
  * Otherwise (i.e., first time we create the record or if the system-id
  * changed) we create a new record.
+ *
+ * Sets '*chassis_rec' to point to the local chassis record.
+ * Returns true if this record was already in the database, false if it was
+ * just inserted.
  */
-static const struct sbrec_chassis *
+static bool
 chassis_get_record(struct ovsdb_idl_txn *ovnsb_idl_txn,
                    struct ovsdb_idl_index *sbrec_chassis_by_name,
-                   const char *chassis_id)
+                   const char *chassis_id,
+                   const struct sbrec_chassis **chassis_rec)
 {
-    const struct sbrec_chassis *chassis_rec =
-        chassis_lookup_by_name(sbrec_chassis_by_name, chassis_id);
+    const struct sbrec_chassis *chassis =
+        chassis = chassis_lookup_by_name(sbrec_chassis_by_name, chassis_id);
 
-    if (!chassis_rec && ovnsb_idl_txn) {
+    if (!chassis && ovnsb_idl_txn) {
         /* Create the chassis record. */
         VLOG_DBG("Could not find Chassis, will create it: %s", chassis_id);
-        chassis_rec = sbrec_chassis_insert(ovnsb_idl_txn);
+        *chassis_rec = sbrec_chassis_insert(ovnsb_idl_txn);
+        return false;
     }
-    return chassis_rec;
+
+    *chassis_rec = chassis;
+    return true;
 }
 
-/* Update a Chassis record based on the config in the ovs config. */
-static void
+/* Update a Chassis record based on the config in the ovs config.
+ * Returns true if 'chassis_rec' was updated, false otherwise.
+ */
+static bool
 chassis_update(const struct sbrec_chassis *chassis_rec,
                struct ovsdb_idl_txn *ovnsb_idl_txn,
                const struct ovs_chassis_cfg *ovs_cfg,
                const char *chassis_id,
                const struct sset *transport_zones)
 {
+    bool updated = false;
+
     if (strcmp(chassis_id, chassis_rec->name)) {
         sbrec_chassis_set_name(chassis_rec, chassis_id);
+        updated = true;
     }
 
     if (strcmp(ovs_cfg->hostname, chassis_rec->hostname)) {
         sbrec_chassis_set_hostname(chassis_rec, ovs_cfg->hostname);
+        updated = true;
     }
 
-    if (chassis_external_ids_changed(ovs_cfg->bridge_mappings,
+    if (chassis_other_config_changed(ovs_cfg->bridge_mappings,
                                      ovs_cfg->datapath_type,
                                      ovs_cfg->cms_options,
+                                     ovs_cfg->monitor_all,
                                      ovs_cfg->chassis_macs,
                                      &ovs_cfg->iface_types,
                                      ovs_cfg->is_interconn,
                                      chassis_rec)) {
-        struct smap ext_ids;
+        struct smap other_config;
 
-        smap_clone(&ext_ids, &chassis_rec->external_ids);
-        chassis_build_external_ids(&ext_ids, ovs_cfg->bridge_mappings,
+        smap_clone(&other_config, &chassis_rec->other_config);
+        chassis_build_other_config(&other_config, ovs_cfg->bridge_mappings,
                                    ovs_cfg->datapath_type,
                                    ovs_cfg->cms_options,
+                                   ovs_cfg->monitor_all,
                                    ovs_cfg->chassis_macs,
                                    ds_cstr_ro(&ovs_cfg->iface_types),
                                    ovs_cfg->is_interconn);
-        sbrec_chassis_verify_external_ids(chassis_rec);
-        sbrec_chassis_set_external_ids(chassis_rec, &ext_ids);
-        smap_destroy(&ext_ids);
+        sbrec_chassis_verify_other_config(chassis_rec);
+        sbrec_chassis_set_other_config(chassis_rec, &other_config);
+        /* TODO(lucasagomes): Continue writing the configuration to the
+         * external_ids column for backward compatibility with the current
+         * systems, this behavior should be removed in the future. */
+        sbrec_chassis_set_external_ids(chassis_rec, &other_config);
+        smap_destroy(&other_config);
+        updated = true;
     }
 
     update_chassis_transport_zones(transport_zones, chassis_rec);
@@ -481,7 +533,7 @@ chassis_update(const struct sbrec_chassis *chassis_rec,
                                 &ovs_cfg->encap_ip_set, ovs_cfg->encap_csum,
                                 chassis_rec);
     if (!tunnels_changed) {
-        return;
+        return updated;
     }
 
     struct sbrec_encap **encaps;
@@ -493,6 +545,7 @@ chassis_update(const struct sbrec_chassis *chassis_rec,
                              ovs_cfg->encap_csum, &n_encap);
     sbrec_chassis_set_encaps(chassis_rec, encaps, n_encap);
     free(encaps);
+    return true;
 }
 
 /* Returns this chassis's Chassis record, if it is available. */
@@ -512,20 +565,24 @@ chassis_run(struct ovsdb_idl_txn *ovnsb_idl_txn,
         return NULL;
     }
 
-    const struct sbrec_chassis *chassis_rec =
-        chassis_get_record(ovnsb_idl_txn, sbrec_chassis_by_name,
-                           chassis_id);
+    const struct sbrec_chassis *chassis_rec = NULL;
+    bool existed = chassis_get_record(ovnsb_idl_txn, sbrec_chassis_by_name,
+                                      chassis_id, &chassis_rec);
 
     /* If we found (or created) a record, update it with the correct config
      * and store the current chassis_id for fast lookup in case it gets
      * modified in the ovs table.
      */
     if (chassis_rec && ovnsb_idl_txn) {
-        chassis_update(chassis_rec, ovnsb_idl_txn, &ovs_cfg, chassis_id,
-                       transport_zones);
-        ovsdb_idl_txn_add_comment(ovnsb_idl_txn,
-                                  "ovn-controller: registering chassis '%s'",
-                                  chassis_id);
+        bool updated = chassis_update(chassis_rec, ovnsb_idl_txn, &ovs_cfg,
+                                      chassis_id, transport_zones);
+
+        if (!existed || updated) {
+            ovsdb_idl_txn_add_comment(ovnsb_idl_txn,
+                                      "ovn-controller: %s chassis '%s'",
+                                      !existed ? "registering" : "updating",
+                                      chassis_id);
+        }
     }
 
     ovs_chassis_cfg_destroy(&ovs_cfg);
@@ -538,7 +595,7 @@ chassis_get_mac(const struct sbrec_chassis *chassis_rec,
                 struct eth_addr *chassis_mac)
 {
     const char *tokens
-        = get_chassis_mac_mappings(&chassis_rec->external_ids);
+        = get_chassis_mac_mappings(&chassis_rec->other_config);
     if (!tokens[0]) {
        return false;
     }
