@@ -82,6 +82,11 @@ struct gen_opts_map {
 #define DHCP_OPT_TFTP_SERVER_ADDRESS \
     DHCP_OPTION("tftp_server_address", 150, "ipv4")
 
+#define DHCP_OPT_ARP_CACHE_TIMEOUT \
+    DHCP_OPTION("arp_cache_timeout", 35, "uint32")
+#define DHCP_OPT_TCP_KEEPALIVE_INTERVAL \
+    DHCP_OPTION("tcp_keepalive_interval", 38, "uint32")
+
 static inline uint32_t
 gen_opt_hash(char *opt_name)
 {
@@ -178,8 +183,11 @@ struct dhcp_opt6_header {
 #define DHCPV6_OPT_SERVER_ID_CODE        2
 #define DHCPV6_OPT_IA_NA_CODE            3
 #define DHCPV6_OPT_IA_ADDR_CODE          5
+#define DHCPV6_OPT_STATUS_CODE           13
 #define DHCPV6_OPT_DNS_SERVER_CODE       23
 #define DHCPV6_OPT_DOMAIN_SEARCH_CODE    24
+#define DHCPV6_OPT_IA_PD                 25
+#define DHCPV6_OPT_IA_PREFIX             26
 
 #define DHCPV6_OPT_SERVER_ID \
     DHCP_OPTION("server_id", DHCPV6_OPT_SERVER_ID_CODE, "mac")
@@ -258,6 +266,22 @@ struct ovs_nd_route_info {
 };
 BUILD_ASSERT_DECL(ND_ROUTE_INFO_OPT_LEN == sizeof(struct ovs_nd_route_info));
 
+OVS_PACKED(
+struct dhcpv6_opt_ia_prefix {
+    struct dhcpv6_opt_header opt;
+    ovs_be32 plife_time;
+    ovs_be32 vlife_time;
+    uint8_t plen;
+    struct in6_addr ipv6;
+});
+
+OVS_PACKED(
+struct dhcpv6_opt_status {
+    struct dhcpv6_opt_header opt;
+    ovs_be16 status_code;
+    uint8_t msg[];
+});
+
 #define DHCPV6_DUID_LL      3
 #define DHCPV6_HW_TYPE_ETH  1
 
@@ -385,6 +409,17 @@ ipv6_addr_is_routable_multicast(const struct in6_addr *ip) {
     }
 }
 
+static inline bool
+ipv6_addr_is_host_zero(const struct in6_addr *prefix,
+                       const struct in6_addr *mask)
+{
+    /* host-bits-non-zero <=> (prefix ^ mask) & prefix. */
+    struct in6_addr tmp = ipv6_addr_bitxor(prefix, mask);
+
+    tmp = ipv6_addr_bitand(&tmp, prefix);
+    return ipv6_is_zero(&tmp);
+}
+
 #define IPV6_EXT_HEADER_LEN 8
 struct ipv6_ext_header {
     uint8_t ip6_nxt_proto;
